//+------------------------------------------------------------------+
//|                                                   XmlElement.mqh |
//|                                                   yu-sha@ukr.net |
//+------------------------------------------------------------------+

//-----------------------------------------------------------------------------
//                                  CXmlElement                               !
//-----------------------------------------------------------------------------

class CXmlElement 
{
  private:

    string         FName;
    CXmlAttribute* FAttributes[];
    CXmlElement   *FElements[];
    string         FText;
    CXmlElement*   FParent;

  public:
  
    // Constructor procedures
    
    void  CXmlElement ();
    void ~CXmlElement ();
    void Init (const string aName, const CXmlElement* aParent=NULL, const string aText="");
    void CopyTo (CXmlElement &aDst);
    virtual void Clear ();
    
    // Main service procedures
    
    string GetName () const;
    void SetName (const string aName);
    string GetText () const;
    void SetText (const string aText);
    CXmlElement* GetParent () const;
    void SetParent (const CXmlElement* aParent);
    
    // Attribute service procedures
  
    int GetAttributeCount () const;
    int GetAttributeIndex (CXmlAttribute* aAttr) const;
    CXmlAttribute* GetAttribute (const string aName) const;
    CXmlAttribute* GetAttribute (int aPos) const;
    string GetAttributeValue (const string aName) const;
    
    CXmlAttribute* AttributeInsertAt (CXmlAttribute* aAttr, int aPos);
    CXmlAttribute* AttributeAdd (CXmlAttribute* aAttr);
    CXmlAttribute* AttributeInsertAfter (CXmlAttribute* aAfter, CXmlAttribute* aAttr);
    CXmlAttribute* AttributeInsertBefore (CXmlAttribute* aBefore, CXmlAttribute* aAttr);
    CXmlAttribute* AttributeRemove (CXmlAttribute* aAttr);
    CXmlAttribute* AttributeRemove (int aPos);
    void AttributeDelete (CXmlAttribute* aAttr);
    void AttributeDelete (int aPos);
    void AttributeDeleteAll ();
  
    // Child service procedures

    int GetChildCount() const;
    int GetChildIndex (CXmlElement* aElement) const;
    CXmlElement* GetChild (const string aName) const;
    CXmlElement* GetChild (int aPos) const;
    string GetChildText (const string aName) const;

    CXmlElement* ChildInsertAt (CXmlElement* aElement, int aPos);
    CXmlElement* ChildAdd (CXmlElement* aElement);
    CXmlElement* ChildInsertAfter (CXmlElement* aAfter, CXmlElement* aElement);
    CXmlElement* ChildInsertBefore (CXmlElement* aBefore, CXmlElement* aElement);
    CXmlElement* ChildRemove (CXmlElement* aElement);
    CXmlElement* ChildRemove (int aPos);
    void ChildDelete (CXmlElement* aElement);
    void ChildDelete (int aPos);
    void ChildDeleteAll ();
  
    string GetXml(int aLevel);
};

//--------------------------------------------------------------------------------/
//                              CXmlElement :: implementation                     /
//--------------------------------------------------------------------------------/
   
  // Constructor procedures

void CXmlElement::CXmlElement () 
{
  FParent = NULL;
};

void CXmlElement::~CXmlElement() 
{
  Clear ();
};
  
void CXmlElement::Init (const string aName, const CXmlElement* aParent=NULL, const string aText="")
{
  Clear ();
  SetName (aName);
  SetParent (aParent);
  SetText (aText);
};

void CXmlElement::CopyTo (CXmlElement &aDst)
{
  aDst.Clear();
  
  aDst.FName = FName;
  aDst.FText = FText;
  aDst.FParent = NULL;

  for (int i=0; i<ArraySize(FAttributes); ++i) 
    aDst.AttributeAdd (FAttributes[i].Clone());

  for (int i=0; i<ArraySize(FElements); ++i) {
    CXmlElement* aItem = new CXmlElement;
    FElements[i].CopyTo (aItem);
    aDst.ChildAdd (aItem);
  };
};

void CXmlElement::Clear ()
{
  FName = "";
  FText = "";
  AttributeDeleteAll ();
  ChildDeleteAll ();
};

  // Main service procedures

string CXmlElement::GetName () const
{
  return FName;
};

void CXmlElement::SetName (const string aName)
{
  FName = aName;
};

string CXmlElement::GetText () const
{
  return FText;
};

void CXmlElement::SetText (const string aText)
{
  FText = aText;
};

CXmlElement* CXmlElement::GetParent () const
{
  return FParent;
};

void CXmlElement::SetParent (const CXmlElement* aParent)
{
  FParent = aParent;
};

  // Attribute service procedures

int CXmlElement::GetAttributeCount () const
{
  return ArraySize (FAttributes);
};

int CXmlElement::GetAttributeIndex (CXmlAttribute* aAttr) const
{
  int i = 0;
  while ( (i < ArraySize(FAttributes)) && (FAttributes[i] != aAttr) )
    ++i;
  
  return ( i<ArraySize(FAttributes) ? i : INT_MAX );  
};

CXmlAttribute* CXmlElement::GetAttribute (const string aName) const
{
  int i=0;
  while ( (i<ArraySize(FAttributes)) && (aName!=FAttributes[i].GetName()) ) 
    ++i;
    
  if (i<ArraySize(FAttributes)) 
    return FAttributes[i]; 
  else 
    return NULL;
};

CXmlAttribute* CXmlElement::GetAttribute (int aPos) const
{
  return FAttributes[aPos];
};

string CXmlElement::GetAttributeValue (const string aName) const
{
  int i=0;
  while ( (i<ArraySize(FAttributes)) && (aName!=FAttributes[i].GetName()) ) 
    ++i;
  return ( i<ArraySize(FAttributes) ? FAttributes[i].GetValue() : "" );
};

CXmlAttribute* CXmlElement::AttributeInsertAt (CXmlAttribute* aAttr, int aPos)
{
  ArrayResize (FAttributes, ArraySize(FAttributes)+1);
  for (int i=ArraySize(FAttributes)-1; i>(aPos); --i)  
    FAttributes[i] = FAttributes[i-1];
  FAttributes[aPos] = aAttr; 
  return aAttr;
};

CXmlAttribute* CXmlElement::AttributeAdd (CXmlAttribute* aAttr)
{
  return AttributeInsertAt (aAttr, GetAttributeCount());
};

CXmlAttribute* CXmlElement::AttributeInsertAfter (CXmlAttribute* aAfter, CXmlAttribute* aAttr)
{
  return AttributeInsertAt (aAttr, GetAttributeIndex(aAfter)+1);
};

CXmlAttribute* CXmlElement::AttributeInsertBefore (CXmlAttribute* aBefore, CXmlAttribute* aAttr) 
{
  return AttributeInsertAt (aAttr, GetAttributeIndex(aBefore));
};

CXmlAttribute* CXmlElement::AttributeRemove (CXmlAttribute* aAttr)
{
  return AttributeRemove (GetAttributeIndex(aAttr));
};

CXmlAttribute* CXmlElement::AttributeRemove (int aPos)
{
  CXmlAttribute* attr = FAttributes[aPos];
  
  for (int i=aPos; i<ArraySize(FAttributes)-1; ++i)  
    FAttributes[i] = FAttributes[i+1];

  ArrayResize (FAttributes, ArraySize(FAttributes)-1);
  
  return attr;
};

void CXmlElement::AttributeDelete (CXmlAttribute* aAttr) 
{
  delete AttributeRemove (aAttr);
};

void CXmlElement::AttributeDelete (int aPos) 
{
  delete AttributeRemove (aPos);
};

void CXmlElement::AttributeDeleteAll ()
{
  for (int i=ArraySize(FAttributes)-1; i>=0; i--) 
    delete FAttributes[i];
  ArrayResize(FAttributes,0);
};
  
  // Child service procedures
  
int CXmlElement::GetChildCount() const
{
  return ArraySize (FElements);
};

int CXmlElement::GetChildIndex (CXmlElement* aElement) const
{
  int i = 0;
  while ( (i < ArraySize(FElements)) && (FElements[i] != aElement) )
    ++i;
  
  return ( i<ArraySize(FElements) ? i : INT_MAX );  
};

CXmlElement* CXmlElement::GetChild (const string aName) const
{
  int i=0;
  while ( (i<ArraySize(FElements)) && (aName!=FElements[i].GetName()) ) 
    ++i;
    
  if (i<ArraySize(FElements)) 
    return FElements[i]; 
  else 
    return NULL;
};

CXmlElement* CXmlElement::GetChild (int aPos) const
{
  return FElements[aPos];
};

string CXmlElement::GetChildText (const string aName) const
{
  CXmlElement *child = GetChild (aName);
  return child!=NULL ? child.GetText() : "";
};

CXmlElement* CXmlElement::ChildInsertAt (CXmlElement* aElement, int aPos)
{
  ArrayResize (FElements, ArraySize(FElements)+1);

  for (int i=ArraySize(FElements)-1; i>(aPos); --i)  
    FElements[i] = FElements[i-1];

  FElements[aPos] = aElement; 
  aElement.SetParent(GetPointer(this));  

  return aElement;
};

CXmlElement* CXmlElement::ChildAdd (CXmlElement* aElement)
{
  return ChildInsertAt (aElement, GetChildCount());
};

CXmlElement* CXmlElement::ChildInsertAfter (CXmlElement* aAfter, CXmlElement* aElement)
{
  return ChildInsertAt (aElement, GetChildIndex (aAfter) + 1);
};

CXmlElement* CXmlElement::ChildInsertBefore (CXmlElement* aBefore, CXmlElement* aElement)
{
  return ChildInsertAt (aElement, GetChildIndex (aBefore));
};

CXmlElement* CXmlElement::ChildRemove (CXmlElement* aElement)
{
  return ChildRemove (GetChildIndex(aElement));
};

CXmlElement* CXmlElement::ChildRemove (int aPos)
{
  CXmlElement* child = FElements[aPos];
  child.SetParent(NULL);
  
  for (int i=aPos; i<ArraySize(FElements)-1; ++i)  
    FElements[i] = FElements[i+1];

  ArrayResize (FElements, ArraySize(FElements)-1);
  return child;
};

void CXmlElement::ChildDelete (CXmlElement* aElement)
{
  delete ChildRemove (aElement);
};

void CXmlElement::ChildDelete (int aPos)
{
  delete ChildRemove (aPos);
};

void CXmlElement::ChildDeleteAll ()
{
  for (int i=ArraySize(FElements)-1; i>=0; i--) 
    delete FElements[i];
  ArrayResize(FElements,0);
};

string CXmlElement::GetXml (int aLevel) {
  string t="";;
  for (int i=0; i<aLevel; ++i)
    t += "\t";
  string s;
  s = t + "<" + FName;
  for (int i=0; i<ArraySize(FAttributes); i++) 
    StringAdd(s, " "+FAttributes[i].GetName()+"=\""+TextPreProcess(FAttributes[i].GetValue())+"\"");
  if ((ArraySize(FElements)==0) && (FText=="")) 
    StringAdd(s,"/>");
  else {
    StringAdd(s,">");
    for (int i=0; i<ArraySize(FElements); i++)
      StringAdd(s,"\r\n"+FElements[i].GetXml(aLevel+1));
    if ((ArraySize(FElements)>0) && (FText!=""))
      StringAdd(s,"\r\n");
//    StringAdd(s,TextPreProcess(FText));
      StringAdd(s,FText);
      FText="*"+FText+"*";
    if ((ArraySize(FElements)>0))
      StringAdd(s,"\r\n"+t);
    StringAdd(s,"</"+FName+">");
  }
  return(s);
};

string TextPreProcess(const string s) {
//  " & < >  &quote, ...
  return(s);
};
